@@ -40,13 +40,14 @@ class Mixer:
     ) -> bool:
         """Check if the solution_matrix spans target vector values. If not, no chance of a successful mixture here!"""
         difference = solution_matrix - target_vector
+        difference = difference.round(12)  # numerical solution has very tiny errors
         return not any([all(v > 0) or all(v < 0) for v in difference.T])
 
     def _calculate_mix(
         self,
         target: Solution,
         solution_indices: list = None,
-        tolerance: float = 1e-3,
+        tolerance: float = 1e-4,
         min_fraction: float = 0,
     ) -> list:
         """Calculate mixture of solution (solution matrix rows) that combine to achieve the target solution
@@ -62,7 +63,7 @@ class Mixer:
         if not self._is_plausible(A, b):
             return np.nan
         x, err = nnls(A.T, b, maxiter=1e3)
-        x.round(12)  # numerical solution has very tiny errors
+        x = x.round(12)  # numerical solution has very tiny errors
         if err > tolerance:
             return np.nan
         if np.logical_and(x > 0, x < min_fraction).any():
@@ -94,10 +95,10 @@ class Mixer:
             components = self.components
         target_matrix = np.zeros((len(components),))
         for m, c in enumerate(components):
-            if c in target.solute_dict:
-                target_matrix[m] = target.solute_dict[c] * volume
-            elif c in target.solvent_dict:
-                target_matrix[m] = target.solvent_dict[c] * volume
+            if c in target.solutes:
+                target_matrix[m] = target.solutes[c] * volume
+            elif c in target.solvent:
+                target_matrix[m] = target.solvent[c] * volume
         return target_matrix.T
 
     def mix(
@@ -195,6 +196,8 @@ class Mixer:
         Returns:
             np.ndarray: adjacency matrix describing all volume transfers to achieve target solutions
         """
+        self.__solved = False
+
         graph = np.zeros((len(self.solutions), len(self.solutions)))
         self.availableidx = self.stock_idx.copy()
         generation = 0
@@ -202,11 +205,13 @@ class Mixer:
         n_remaining_lastiter = np.inf
         while n_remaining > 0:
             if generation > max_generations or n_remaining == n_remaining_lastiter:
-                print("Could not find a solution")
+                error_string = (
+                    "Could not find a solution for the following solutions:\n"
+                )
                 for i, s in enumerate(self.solutions):
                     if i not in self.availableidx:
-                        print(s)
-                break
+                        error_string += f"\t{s}\n"
+                raise Exception(error_string)
             n_remaining_lastiter = n_remaining
 
             for i in range(len(self.solutions)):
@@ -226,6 +231,8 @@ class Mixer:
                     self.availableidx.append(i)
                     n_remaining -= 1
             generation += 1
+
+        self.__solved = True
         return graph
 
     def solve(
@@ -297,7 +304,6 @@ class Mixer:
             this_gen = {node: transfers[node] for node in gen if node in transfers}
             if len(this_gen) > 0:
                 self.transfers_per_generation.append(this_gen)
-        self.__solved = True
 
     ### publishing methods
     def _check_if_solved(self):
@@ -443,8 +449,8 @@ class Weigher:
         # organize target solution into a matrix of total mols desired of each component
         target_matrix = np.zeros((len(self.components),))
         for m, component in enumerate(self.components):
-            if component in target.solute_dict:
-                target_matrix[m] = target.solute_dict[component] * volume
+            if component in target.solutes:
+                target_matrix[m] = target.solutes[component] * volume
         return target_matrix.T
 
     def _filter_powders(self, target: Solution) -> list:
@@ -509,9 +515,9 @@ class Weigher:
             weights (dict): {powder:mass(g)} dictionary. Powders must be present in this `Weigher`, and can be referred to by the `Powder.alias` or `Powder.formula` attributes
             volume (float): volume (L) of solvent powder is dispersed into
             solvent (str): Formula string for the solvent. ie "IPA", "DMF9_DMSO1", "EtOH0.5_IPA0.5".
-            molarity (float, optional): Value to set molarity of resulting Solution. This can be a directly set float value, or the name of a single component
-                of the resulting `Solution` that molarity will be normalized to. Note that this only affects the string representation of the `Solution.solutes`,
-                the actual `Solution` should be equivalent regardless of your choice here.
+            molarity (float, optional): Value to set molarity of resulting Solution. This can be a directly set float value, the name of a single component
+                of the resulting `Solution` that molarity will be normalized to, or a list of names of components the sum of which will be used as the molarity.
+                Note that this only affects the molarity value and string representation of the `Solution`, the actual `Solution` will be equivalent regardless of your choice here.
         Returns:
             Solution: solution object resulting from the powder mixing
         """
@@ -520,19 +526,27 @@ class Weigher:
             m = self._lookup_powder(powder)
             v += self.matrix[m] * mass / volume  # v is in units of mol/L
 
-        if type(molarity) is str:
+        if isinstance(molarity, str):
             if molarity not in self.components:
                 raise ValueError(
                     f"Cannot set molarity to {molarity} - this component does not exist in the resulting Solution!"
                 )
             molarity = v[self.components.index(molarity)]
+        elif isinstance(molarity, list):
+            temp = 0
+            for m in molarity:
+                if m not in self.components:
+                    raise ValueError(
+                        f"Cannot set molarity to {molarity} - this component does not exist in the resulting Solution!"
+                    )
+                temp += v[self.components.index(m)]
+            molarity = temp
+
         elif molarity is None:
             molarity = v.max()
 
-        v /= molarity
-        solutes = components_to_name(
-            components={c: v for c, v in zip(self.components, v) if v > 0}
-        )
+        # v *= molarity
+        solutes = {c: v for c, v in zip(self.components, v) if v > 0}
         return Solution(solutes=solutes, solvent=solvent, molarity=molarity)
 
 
@@ -540,14 +554,14 @@ def _order_components(solutions: list, components: list):
     """Order the components by their average order across the solutions. Solvents come after solutes"""
     component_rank = [[] for _ in components]
     for s in solutions:
-        for c in s.solute_dict:
-            if c in s.solutes:
-                component_rank[components.index(c)].append(s.solutes.index(c))
-        for c in s.solvent_dict:
-            if c in s.solvent:
-                component_rank[components.index(c)].append(
-                    s.solvent.index(c) + 1000
-                )  # penalize solvents so they come after solutes
+        this_solutes = list(s.solutes.keys())
+        this_solvent = list(s.solvent.keys())
+        for c in s.solutes:
+            component_rank[components.index(c)].append(this_solutes.index(c))
+        for c in s.solvent:
+            component_rank[components.index(c)].append(
+                this_solvent.index(c) + 1000
+            )  # penalize solvents so they come after solutes
     avg_component_rank = [np.mean(ranks) for ranks in component_rank]
     return [components[i] for i in np.argsort(avg_component_rank)]
 
@@ -573,7 +587,7 @@ def _solutions_to_matrix(solutions: list, components: list = None) -> tuple:
     if components is None:
         components = set()
         for s in solutions:
-            components.update(s.solute_dict.keys(), s.solvent_dict.keys())
+            components.update(s.solutes.keys(), s.solvent.keys())
         components = list(
             components
         )  # sets are not order-preserving, lists are - just safer this way
@@ -587,48 +601,49 @@ def _solutions_to_matrix(solutions: list, components: list = None) -> tuple:
     solvent_idx = set()
     for m, s in enumerate(solutions):
         for n, c in enumerate(components):
-            if c in s.solute_dict:
-                solution_matrix[m, n] = s.solute_dict[c]
-            elif c in s.solvent_dict:
-                solution_matrix[m, n] = s.solvent_dict[c]
+            if c in s.solutes:
+                solution_matrix[m, n] = s.solutes[c]
+            elif c in s.solvent:
+                solution_matrix[m, n] = s.solvent[c]
                 solvent_idx.add(n)
     solvent_idx = list(solvent_idx)
 
     return solution_matrix, solvent_idx, components
 
 
-def interpolate(solutions: list, steps: int) -> list:
+def interpolate(endpoints: list, divisor: int) -> list:
     """Generate a list of solutions that are a linear interpolation between the given solutions
 
     Args:
-        solutions (list): List of Solution objects to be interpolated between
+        endpoints (list): List of Solution objects to be interpolated between
         steps (int): number of steps to interpolate between endpoint solutions. 1 will return the original list, 2 will split into 50% increments, 3 split into 33% increments, etc.
 
     Returns:
         list: list of unique Solution objects resulting from the interpolation
     """
+    if divisor <= 0:
+        raise ValueError("Divisor must be greater than 0!")
+    if not isinstance(divisor, int):
+        raise ValueError("Divisor must be an integer!")
 
-    solution_matrix, solvent_idx, components = _solutions_to_matrix(solutions)
+    solution_matrix, solvent_idx, components = _solutions_to_matrix(endpoints)
     solvent_components = [components[i] for i in solvent_idx]
-    solution_idx = list(range(len(solutions)))
+    solution_idx = list(range(len(endpoints)))
     tweened_solutions = []
-    for solution_indices in itt.combinations_with_replacement(solution_idx, steps):
+    for solution_indices in itt.combinations_with_replacement(solution_idx, divisor):
         svector = np.mean([solution_matrix[i] for i in solution_indices], axis=0)
-        molarity = np.mean([solutions[i].molarity for i in solution_indices])
-        solutes = components_to_name(
-            {
-                c: v / molarity
-                for c, v in zip(components, svector)
-                if v > 0 and c not in solvent_components
-            }
-        )
-        solvent = components_to_name(
-            {
-                c: v
-                for c, v in zip(components, svector)
-                if v > 0 and c in solvent_components
-            }
-        )
+        molarity = np.mean([endpoints[i].molarity for i in solution_indices])
+        solutes = {
+            c: v / molarity
+            for c, v in zip(components, svector)
+            if v > 0 and c not in solvent_components
+        }
+        solvent = {
+            c: v
+            for c, v in zip(components, svector)
+            if v > 0 and c in solvent_components
+        }
+
         new_solution = Solution(solutes=solutes, solvent=solvent, molarity=molarity)
         if new_solution not in tweened_solutions:
             tweened_solutions.append(new_solution)
